package cz.esw.profiling.tsp;

import cz.esw.profiling.ga.Crossover;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

/**
 * At first it randomly selects one {@code cut} position on the first parent. The offspring is then generated by
 * appending all cities (preserving their order) from the second parent not contained in the segment before the cut on
 * the first parent.
 *
 * @author Marek Cuchý
 */
public class SinglePointCrossover implements Crossover<TspIndividual> {

	private final Random rnd;

	public SinglePointCrossover(Random rnd) {
		this.rnd = rnd;
	}

	@Override
	public TspIndividual perform(TspIndividual parent1, TspIndividual parent2) {
		Integer[] order1 = parent1.getOrder();
		Integer[] order2 = parent2.getOrder();

		int split = rnd.nextInt(order1.length - 1) + 1; // split point securing at least one element from both parents

		Integer[] offspring = new Integer[order1.length];

		Set<Integer> contained = new HashSet<>();

		//add cities from parent1 until the split is reached
		int i = 0;
		for (; i < split; i++) {
			int city = order1[i];
			offspring[i] = city;
			contained.add(city);
		}

		//add cities from parent2 not already in the offspring
		for (int j = 0; j < order2.length; j++) {
			int city = order2[j];
			if (!contained.contains(city)) {
				offspring[i++] = city;
			}
		}
		return new TspIndividual(parent1.getTsp(), offspring);
	}
}
